/* 
 * Copyright (c)2007-2008, Dustin Spicuzza
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * The name of the Dustin Spicuzza may not be used to endorse or promote 
 *       products derived from this software without specific prior written 
 *       permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Dustin Spicuzza ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Dustin Spicuzza BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;

namespace Panes {


	/// <summary>
	/// This implements a Pane that will hold a control to be displayed. 
	/// </summary>
	public class Pane : PaneBase {

		private gradientLabel titleLabel = new gradientLabel();
		private Label controlBox = new Label();

		/// <summary>
		/// This event is called when the pane is closed by the user.
		/// </summary>
		public event EventHandler PaneClosing = null;

		public Pane(Control c, string text) {
			
			Initialize();

			this.Text = text;
			this.ContainedControl = c;
		}

		public Pane() {
			Initialize();		
		}

		private void Initialize() {

			// do my initialization here, pretty much the same thing 
			// as InitializeComponent :) 


			this.Dock = DockStyle.Fill;
			this.IsRoot = false;

			titleLabel.BeginColor = System.Drawing.SystemColors.GradientActiveCaption;
			titleLabel.EndColor = System.Drawing.SystemColors.GradientInactiveCaption;
			titleLabel.Name = "titleLabel";
			titleLabel.AutoEllipsis = true;

			titleLabel.MouseMove += new MouseEventHandler(titleLabel_MouseMove);
			titleLabel.MouseDown += new MouseEventHandler(titleLabel_MouseDown);
			titleLabel.MouseUp += new MouseEventHandler(titleLabel_MouseUp);

			Controls.Add(titleLabel);

			titleLabel.Dock = DockStyle.Top;

			controlBox.AutoSize = true;
			controlBox.Text = "X";
			controlBox.Font = new Font("Microsoft Sans Serif", 10F, FontStyle.Bold);
			controlBox.BackColor = System.Drawing.SystemColors.GradientInactiveCaption;
			controlBox.MouseEnter += new EventHandler(controlBox_MouseEnter);
			controlBox.MouseLeave += new EventHandler(controlBox_MouseLeave);
			controlBox.Click += new EventHandler(controlBox_Click);

			Controls.Add(controlBox);

			OnResize(null);
		}

		private Control containedControl = null;

		public Control ContainedControl {
			get { return containedControl; }
			set {

				if ( containedControl != null )
					containedControl.Parent = null;
				
				containedControl = value;

				if ( containedControl != null ) {
					Controls.Add(containedControl);
					containedControl.BringToFront();
					containedControl.Dock = DockStyle.Fill;
				}
			}
		}


		bool moving = false;
		bool isClosed = false;

		/// <summary>
		/// Set to true if the Pane has been closed
		/// </summary>
		public bool IsClosed {
			get { return isClosed; }
		}

		void titleLabel_MouseUp(object sender, MouseEventArgs e) {
			if (e.Button == MouseButtons.Left)
				moving = false;
		}

		void titleLabel_MouseDown(object sender, MouseEventArgs e) {
			if ( e.Button == MouseButtons.Left )
				moving = true;
		}

		void titleLabel_MouseMove(object sender, MouseEventArgs e) {
			if ( moving ) {

				moving = false;

				Form form = this.FindForm();
				Size sz = this.Size;

				// this must be called after we obtain the parent form
				DoRelocation();

				// create a new form, attach self to it
				FloatingPane fpane = new FloatingPane(this, form);

				// position it, and size it
				fpane.Size = sz;

				fpane.StartPosition = FormStartPosition.Manual;
				fpane.DesktopLocation = new Point(Control.MousePosition.X - fpane.Width/2, Control.MousePosition.Y - SystemInformation.ToolWindowCaptionHeight/2);
				fpane.Focus();
				fpane.Show();
			}
		}
		/// <summary>
		/// This tries to reorganize the container above it when it is removed, in certain
		/// circumstances.
		/// </summary>
		private void DoRelocation() {

			Type t = Parent.GetType();

			// remove the splitter first
			if ( mySplitter != null ) {
				mySplitter.Parent = null;
				mySplitter.Dispose();
				mySplitter = null;
			}

			// first, save reference to parent. 
			PaneContainer parent = t == typeof(PaneContainer) ? Parent as PaneContainer : null;

			// second
			this.Parent = null;
			
			// finally, call reorganize if the parent isn't null
			if ( parent != null )
				parent.DoReorganize();
		}

		/// <summary>
		/// Closes the pane and removes it from the layout
		/// </summary>
		public void Close() {

			// launch this event first
			if ( PaneClosing != null )
				PaneClosing(this, EventArgs.Empty);

			isClosed = true;

			DoRelocation();

			this.Dispose();
		}

		void controlBox_Click(object sender, EventArgs e) {
			Close();
		}

		void controlBox_MouseLeave(object sender, EventArgs e) {
			controlBox.BackColor = System.Drawing.SystemColors.GradientInactiveCaption;
		}

		void controlBox_MouseEnter(object sender, EventArgs e) {
			controlBox.BackColor = System.Drawing.SystemColors.GradientActiveCaption;
		}

		public override bool IsRoot {
			get { return base.IsRoot; }
			set {
				base.IsRoot = value;

				titleLabel.Visible = controlBox.Visible = !value;

				if ( !value )
					controlBox.BringToFront();
				else if (Parent != null)
					Parent.Text = Text;

				// do necessary work
				OnResize(null);
			}
		}


		protected override void OnResize(EventArgs e) {
			base.OnResize(e);

			if ( !IsRoot ) {
				controlBox.Left = Width - ( controlBox.Width * 5 ) / 4;
				controlBox.Top = ( titleLabel.Height - controlBox.Height ) / 2;
			}

		}

		protected override void OnParentChanged(EventArgs e) {
			base.OnParentChanged(e);

			if ( Parent != null && IsRoot )
				Parent.Text = Text;
		}


		public override string Text {
			get {
				return titleLabel.Text;
			}
			set {
				titleLabel.Text = value;
				if ( Parent != null && IsRoot )
					Parent.Text = Text;
			}
		}


		protected override void Dispose(bool disposing) {

			if ( !IsClosed ) {

				if ( PaneClosing != null )
					PaneClosing(this, EventArgs.Empty);

				isClosed = true;
			}
			
			base.Dispose(disposing);
		}
	}
}
